#include <owl\pch.h>
#pragma hdrstop

#include <owl\edit.h>
#include <dos.h>
#include <math.h>
#include "capmsg.rh"

#define CAPTION "Caption Message"
const float secMag = 13.0;
const float minMag = 12.0;
const float hourMag = 8.0;


class TWndw : public TFrameWindow
{
public:
	TWndw(TWindow *parent, const char far *title);
	~TWndw();
protected:
	void SetupWindow();
	void CleanupWindow();
	void EvTimer(UINT);
	void Paint(TDC& dc, bool erase, TRect& rect);
private:
	void CmHelpAbout();
	void DrawHands();
	void GetNewPos();

	struct time t;
	TPoint hourOldPos, hourNewPos;
	TPoint minOldPos, minNewPos;
	TPoint secOldPos, secNewPos;
DECLARE_RESPONSE_TABLE(TWndw);
};

DEFINE_RESPONSE_TABLE1(TWndw, TFrameWindow)
	EV_WM_TIMER,
	EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
END_RESPONSE_TABLE;


TWndw::TWndw(TWindow *parent, const char far *title)
	:	TFrameWindow(parent, title)
{
	AssignMenu(MENU_1);
	GetNewPos();
}


TWndw::~TWndw()
{
}


void TWndw::SetupWindow()
{
	TFrameWindow::SetupWindow();
	SetTimer(1, 1000);
}


void TWndw::CleanupWindow()
{
	KillTimer(1);
}


void TWndw::GetNewPos()
{
	gettime(&t);
	float hourAngle = ((2 * M_PI) / 12.0) * t.ti_hour - M_PI_2;
	float minAngle = ((2 * M_PI) / 60.0) * t.ti_min - M_PI_2;
	float secAngle = ((2 * M_PI) / 60.0) * t.ti_sec - M_PI_2;

	hourNewPos.x = hourMag * cos(hourAngle);
	hourNewPos.y = hourMag * sin(hourAngle);
	minNewPos.x = minMag * cos(minAngle);
	minNewPos.y = minMag * sin(minAngle);
	secNewPos.x = secMag * cos(secAngle);
	secNewPos.y = secMag * sin(secAngle);
}


void TWndw::EvTimer(UINT)
{
	if (IsIconic()) {
		DrawHands();
		char buf[30];
		wsprintf(buf, "%2d:%02d:%02d", t.ti_hour, t.ti_min, t.ti_sec);
		SetCaption(buf);
	} else {
		SetCaption(CAPTION);
		Invalidate();
		UpdateWindow();
	}
}


void TWndw::DrawHands()
{
	TClientDC clientDC(*this);
	hourOldPos = hourNewPos;
	minOldPos = minNewPos;
	secOldPos = secNewPos;
	GetNewPos();

	clientDC.MoveTo(15, 15);
	clientDC.SelectObject(TPen(TColor::Black));
	clientDC.LineTo(hourOldPos.OffsetBy(15, 15));
	clientDC.RestorePen();

	clientDC.MoveTo(15, 15);
	clientDC.SelectObject(TPen(TColor::LtGray));
	clientDC.LineTo(hourNewPos.OffsetBy(15, 15));
	clientDC.RestorePen();

	clientDC.MoveTo(15, 15);
	clientDC.SelectObject(TPen(TColor::Black));
	clientDC.LineTo(minOldPos.OffsetBy(15, 15));
	clientDC.RestorePen();

	clientDC.MoveTo(15, 15);
	clientDC.SelectObject(TPen(TColor::White));
	clientDC.LineTo(minNewPos.OffsetBy(15, 15));
	clientDC.RestorePen();

	clientDC.MoveTo(15, 15);
	clientDC.SelectObject(TPen(TColor::Black));
	clientDC.LineTo(secOldPos.OffsetBy(15, 15));
	clientDC.RestorePen();

	clientDC.MoveTo(15, 15);
	clientDC.SelectObject(TPen(TColor::LtRed));
	clientDC.LineTo(secNewPos.OffsetBy(15, 15));
	clientDC.RestorePen();
}


void TWndw::Paint(TDC& dc, bool /*erase*/, TRect& /*rect*/)
{
	gettime(&t);

	char buf[30];
	wsprintf(buf, "%2d:%02d:%02d", t.ti_hour, t.ti_min, t.ti_sec);

	TRect clientRect = GetClientRect();
	dc.SetTextAlign(TA_CENTER | TA_BASELINE);
	dc.TextOut(clientRect.right / 2, clientRect.bottom / 2, buf);
}


void TWndw::CmHelpAbout()
{
	string msg;
	msg += "When the program is minimized,\n";
	msg += "the clock will be display as caption";

	MessageBox(msg.c_str(), "About Caption Message");
}

///////////////////////////////////////////////////////////////////////////
class TApp : public TApplication
{
public:
	TApp(): TApplication() {}
	void InitMainWindow();
};


void TApp::InitMainWindow()
{
	TFrameWindow *wndw = new TWndw(0, CAPTION);
	SetMainWindow(wndw);
	wndw->SetIcon(this, ICON_1);
	EnableCtl3d();
	nCmdShow = SW_SHOWMINIMIZED;
}


int OwlMain(int, char*[])
{
	return TApp().Run();
}

